libname gfdata 'C:\Public\SAS_GLOBAL_FORUM_2016';

/*libname gfdata 'C:\Users\Public\SAS_Global_Forum_2016';*/
%MACRO FORREST2;
	%let equal=%str(=);
	%let double=%str(%");
	%let n=n;
	%let single=%str(%');
	%let blank=%str( );
	libname gfdata 'C:\Public\SAS_GLOBAL_FORUM_2016';

	PROC SORT
		DATA=&lib..&dsn(KEEP=study outcome type order )
		OUT=WORK.STUDYDATA1 
	;
		BY &studyfield outcome type;
	RUN;

	proc sql;
		create table work.temp1 as
			select distinct &studyfield,weight,order
				from &lib..&dsn;
	quit;

	PROC FREQ DATA = WORK.Studydata1
		ORDER=INTERNAL noprint;
		BY &studyfield;
		TABLES outcome * type /
			NOROW
			NOCOL
			NOPERCENT
			NOCUM
			MEASURES
			SCORES=TABLE
			ALPHA=0.05	OUT=WORK.Cell_statistics_for_outcome
		;
		OUTPUT 		MEASURES
			OUT=WORK.Table_statistics_for_outcomes
		;
	run;

	PROC SORT
		DATA=WORK.CELL_STATISTICS_FOR_OUTCOME(KEEP=COUNT type study outcome)
		OUT=WORK.TMP1
	;
		BY &studyfield outcome type;
	RUN;

	%LET __EG_DROP_LABEL__=;
	%LET __EG_LABEL_OPT__=;

	PROC CONTENTS DATA=WORK.CELL_STATISTICS_FOR_OUTCOME OUT=WORK.TMP1Label(KEEP=LABEL) NOPRINT;
	RUN;

	DATA _NULL_;
		SET WORK.TMP1Label(WHERE=(LABEL NE ""));
		CALL SYMPUT("__EG_DROP_LABEL__", "__EG_LABEL__");
		CALL SYMPUT("__EG_LABEL_OPT__", "LABEL=__EG_LABEL__");
		STOP;
	RUN;

	PROC TRANSPOSE DATA=WORK.TMP1
		OUT=WORK.TRNSSPLITCOLUMNS_0001(LABEL="Split WORK.CELL_STATISTICS_FOR_OUTCOME"
		DROP=__EG_TYPE__ &__EG_DROP_LABEL__ )
		NAME=__EG_TYPE__
		&__EG_LABEL_OPT__
	;
		BY &studyfield outcome;
		ID type;
		VAR COUNT;
	RUN;

	data work.comb1(keep=&studyfield trn pln)
		work.comb2(keep=&studyfield trF plF);
		set WORK.TRNSSPLITCOLUMNS_0001;
		count+1;

		if mod(count,2)= 0 then
			do;
				trN=treatment;
				plN=placebo;
				output comb1;
			end;
		else
			do;
				trF=treatment;
				plF=placebo;
				output comb2;
			end;
	run;

	data work.allcomb;
		merge comb1 comb2 temp1;
		by &studyfield;
		output;
	run;

	proc sql;
		select sum(N) into :total
			from work.table_statistics_for_outcomes;
	quit;

	/*%put Total Count across all studies &total;*/
	/*** ODDS RATIO _RRC2_ L_RRC2 U_RRC2  ***/
	data work.combine;
		merge work.allcomb  work.table_statistics_for_outcomes end=last;
		by &studyfield;
		COMBO=put(N,3.)||'('||put(weight,percent5.)||')';
		x_low=_RROR_/(10**(weight/2));
		x_high=_RROR_*(10**(weight/2));
		perTrF=TrF/(TrN+TrF);
		perTrN=TrN/(TrN+TrF);
		perPlF=PlF/(PlN+PlF);
		perPlN=PlN/(PlN+PlF);
		COMBO=put(N,3.)||'('||put(perTRF,percent5.)||')';
		studytext=trim(left(study))||' '||'('||trim(left(order))||')';
		LABEL N='Subjects'
			Combo='N/(percent)'
			weight='Weight'
			pertrf='(%)Tr Favor'
			pertrn='(%)Tr Adverse'
			perplf='(%)Pl Favor'
			perplN='(%)Pl Adverse'
			trf='Received Treat'
			plf='Received Placebo'
			trN='No Treatment'
			plN='No Placebo'
			_PCORR_='P-Value'
			L_RROR='LCLM'
			U_RROR='UCLM'

			_RRC2_='Odds Ratio'
			studytext='Study';
		format N comma6.
			pertrf percent7.1
			pertrn percent7.1
			perplf percent7.1
			perplN percent7.1
			trf comma6.
			plf comma6.
			trN comma6.
			plN comma6.
			_PCORR_ 6.5
			L_RRC2 6.4
			U_RRC2 6.4
			_RRC2_ 6.4;
		output;
		overall_pertrf+(weight*pertrf);
		overall_pertrn+(weight*pertrn);
		overall_perplf+(weight*perplf);
		overall_perpln+(weight*perpln);
		overall_rror+(weight*_RROR_);
		overall_rrorl+(weight*L_RROR);
		overall_rroru+(weight*U_RROR);
		overall_trf+(weight*trf);
		overall_trn+(weight*trn);
		overall_plf+(weight*plf);
		overall_pln+(weight*pln);
		overpcorr+(weight*_PCORR_);
		over_LRRC2+(weight*L_RRC2);
		over_URRC2+(weight*U_RRC2);
		over_RRC2+(weight*_RRC2_);

		if last=1 then
			do;
				call symputx('overr',overall_rror);
				call symputx('overu',overall_rroru);
				call symputx('overl',overall_rrorl);
				studytext='Overall Effect';
				pertrf=overall_pertrf;
				pertrn=overall_pertrn;
				perplf=overall_perplf;
				perplN=overall_perpln;
				trf=overall_trf;
				plf=overall_plf;
				trN=overall_trn;
				plN=overall_pln;
				_PCORR_=overpcorr;
				L_RRC2=over_LRRC2;
				U_RRC2=over_URRC2;
				_RRC2_=over_RRC2;

				if studytext='Overall Effect' then
					order=9999;
				output;
			end;
	run;

	proc sql noprint;
		create table gfdata.statistics as
			select name as start,label,'$statfmt' as fmtname 
				from dictionary.columns
					where upcase(libname)='WORK' and upcase(memname)='COMBINE' and 
						upcase(type)='NUM' and upcase(name) not in ("ORDER");
	quit;

	proc sql noprint;
		select name into :namelist separated by '#'  
			from dictionary.columns
				where upcase(libname)='WORK' and upcase(memname)='COMBINE' and 
					upcase(type)='NUM' and upcase(name) not in ("ORDER");
	quit;

	/*%put &namelist;*/
	%let totstats=%sysfunc(countw(&namelist,#));

	proc format cntlin=gfdata.statistics lib=gfdata;
		select statfmt;
	run;

	options fmtsearch=(gfdata);
	%put Middle &overr;
	%put Low &overl;
	%put High &overu;

	proc sort data=work.combine;
		by order;
	run;

	%let lsize=&lstats0;
	%let rsize=&rstats0;

	proc template;
		define statgraph Forest_five;
			dynamic studytext 
				%do I=1 %to &LSTATS0;
			&&lstats&I&blank
			%end;

			%do J=1 %to &RSTATS0;
				&&rstats&J&blank
			%end;

		mvar

			%do I=1 %to &LSTATS0;
				lstats&I&blank
			%end;

		%do J=1 %to &RSTATS0;
			rstats&J&blank
		%end;

	pcorr_color odcolor overr overu overl;
	begingraph / designheight=1100 designwidth=1700;
		entrytitle "&top1";
		entrytitle textattrs=(size=8) "&top2";
		layout lattice / columns=1 /*columnweights=(0.33 0.33 0.32)*/;
			layout overlay / walldisplay=none 
				xaxisopts=(display=(ticks tickvalues line)
				type=log logopts=(viewmin=&logmin viewmax=&logmax MinorTicks=true)) 
				y2axisopts=(labelFitPolicy=Split offsetmin=.05 offsetmax=.05 ) 
				yaxisopts=(display=none labelFitPolicy=Split type=discrete 
				reverse=true offsetmin=.05 offsetmax=.05);
				drawtext   textattrs=(family='Times New Roman Uni' color=green size=40 ) {unicode"25BA"x} 
					/ x=&overu y='Overall Effect' anchor=center drawspace=datavalue layer=front width=250 widthunit=pixel;
				drawtext   textattrs=(family='Times New Roman Uni' color=green size=40) {unicode "25C4"x}
					/ x=&overl y='Overall Effect' anchor=center drawspace=datavalue layer=front width=250 widthunit=pixel;
				drawtext  textattrs=(color=Blue weight=bold) "<=====Favors Treatment"/  x=&adjustfactorX1  y=13 anchor=center  layer=front width=40 widthunit=percent drawspace=graphpercent;
				drawtext   textattrs=(color=Blue weight=bold) "Favors Placebo=====>"   / x=&adjustfactorx2 y=13 anchor=center  layer=front width=40 widthunit=percent drawspace=graphpercent;

				/*--Columns on the left--*/
				innermargin / align=left opaque=false  /* pad=(top=2px bottom=2px)*/;
					axistable y='studytext'n value='studytext'n / display=(label) labelposition=max labelattrs=(size=11pt weight=bold)
						valueattrs=(size=10pt weight=bold);

					%do L=1 %to &lsize;
						axistable y='studytext'n value="&&LSTATS&L"n / display=(label) labelposition=max 
							labelattrs=(size=11pt weight=bold) valueattrs=(size=10pt weight=bold color=&&lstat_color&L);
					%end;

				endinnermargin;

				/*--Graph in the middle--*/
				HighLowPlot Y='studytext'n High='U_RROR'n Low='L_RROR'n / primary=true Lineattrs=( Color=CX83838C thickness=2px) LegendLabel="study" NAME="HIGHLOW";
				HighLowPlot Y='studytext'n High='x_high'n Low='x_low'n / type=bar barwidth=.4  display=(fill) FillAttrs=(Color=CXA52A2A) LegendLabel="study" NAME="HIGHLOW1";
				ReferenceLine x=1 / Lineattrs=( Color=black Pattern=1 Thickness=3px);
				ReferenceLine x=0.10 / clip=true Lineattrs=( Color=CX83838C Pattern=2 Thickness=2px);
				ReferenceLine x=0.01 / clip=true Lineattrs=( Color=CX83838C Pattern=2 Thickness=2px);

				%if &logmax > 1.01 %then
					%do;
						ReferenceLine x=10 / Lineattrs=( Color=CX83838C Pattern=2 Thickness=2px);
					%end;
				%else %if &logmax > 10 %then
					%do;
						ReferenceLine x=100 / Lineattrs=( Color=CX83838C Pattern=2 Thickness=2px);
						ReferenceLine x=10 / Lineattrs=( Color=CX83838C Pattern=2 Thickness=2px);
					%end;

				/*--Columns on the Right--*/
				innermargin / align=right opaque=false /*pad=(top=2px bottom=2px)*/;

					%do R=1 %to &rsize;
						axistable y='studytext'n value="&&rstats&R"n / display=(label)  labelposition=max labelattrs=(size=11pt weight=bold) valueattrs=(size=10pt weight=bold color=&&rstat_color&R);
					%end;

				endinnermargin;
			endlayout;

			/*--Second column is empty just to push the third column to the left--*/
		endlayout;
		entryfootnote halign=left textattrs=(size=7)
			"&footnote1";
		entryfootnote halign=left textattrs=(size=7)
			"&footnote2";
	endgraph;
		end;
	run;

%MEND;

%MACRO RENDER;
	%let equal=%str(=);
	%let double=%str(%");
	%let n=n;
	%let single=%str(%');
	%let blank=%str( );

	proc sgrender data=work.combine template=forest_five;
		dynamic studytext="'studytext'n"   
			%do K=1 %to &LSTATS0;
		&blank&&LSTATS&K&equal"&single&&LSTATS&K&single&n"&blank
		%end;

		%do H=1 %to &RSTATS0;
			&blank&&RSTATS&H&equal"&single&&RSTATS&H&single&n"&blank
		%end;
	;
	run;

%MEND;

options mprint nodate nonumber orientation=landscape;

%FORREST2

ODS ESCAPECHAR='~';
ODS GRAPHICS ON / width=12.0in height=8in  ;
ODS RTF path="C:\Public\SAS_Global_Forum_2016" file="sample1.rtf" ;
ODS LISTING gpath='C:\Public\SAS_Global_Forum_2016';

%RENDER
ODS _ALL_ CLOSE;